/** \file inteSiVis.h

    \brief Application for interactive simulation and visualization.

    \see Accompanying articles for more information:
        - http://software.intel.com/en-us/articles/fluid-simulation-for-video-games-part-1/
        - http://software.intel.com/en-us/articles/fluid-simulation-for-video-games-part-2/
        - http://software.intel.com/en-us/articles/fluid-simulation-for-video-games-part-3/
        - http://software.intel.com/en-us/articles/fluid-simulation-for-video-games-part-4/
        - http://software.intel.com/en-us/articles/fluid-simulation-for-video-games-part-5/
        - http://software.intel.com/en-us/articles/fluid-simulation-for-video-games-part-6/
        - http://software.intel.com/en-us/articles/fluid-simulation-for-video-games-part-7/
        - http://software.intel.com/en-us/articles/fluid-simulation-for-video-games-part-8/
        - http://software.intel.com/en-us/articles/fluid-simulation-for-video-games-part-9/
        - http://software.intel.com/en-us/articles/fluid-simulation-for-video-games-part-10/
        - http://software.intel.com/en-us/articles/fluid-simulation-for-video-games-part-11/
        - http://software.intel.com/en-us/articles/fluid-simulation-for-video-games-part-12/
        - http://software.intel.com/en-us/articles/fluid-simulation-for-video-games-part-13/
        - http://software.intel.com/en-us/articles/fluid-simulation-for-video-games-part-14/
        - http://software.intel.com/en-us/articles/fluid-simulation-for-video-games-part-15/

        - http://www.gamasutra.com/view/feature/4164/sponsored_feature_fluid_.php
        - http://www.gamasutra.com/view/feature/4176/sponsored_feature_fluid_.php

        - http://www.mijagourlay.com/

    \author Copyright 2009-2013 Dr. Michael Jason Gourlay; All rights reserved.  Contact me at mijagourlay.com for licensing.

    \mainpage Interactive simulation and visualization of fluids that use vortex elements.

    This code accompanies a series of articles published on the Intel Software Network and Gamasutra.

    \see Accompanying articles for more information:
        - http://software.intel.com/en-us/articles/fluid-simulation-for-video-games-part-1/
        - http://software.intel.com/en-us/articles/fluid-simulation-for-video-games-part-2/
        - http://software.intel.com/en-us/articles/fluid-simulation-for-video-games-part-3/
        - http://software.intel.com/en-us/articles/fluid-simulation-for-video-games-part-4/
        - http://software.intel.com/en-us/articles/fluid-simulation-for-video-games-part-5/
        - http://software.intel.com/en-us/articles/fluid-simulation-for-video-games-part-6/
        - http://software.intel.com/en-us/articles/fluid-simulation-for-video-games-part-7/
        - http://software.intel.com/en-us/articles/fluid-simulation-for-video-games-part-8/
        - http://software.intel.com/en-us/articles/fluid-simulation-for-video-games-part-9/
        - http://software.intel.com/en-us/articles/fluid-simulation-for-video-games-part-10/
        - http://software.intel.com/en-us/articles/fluid-simulation-for-video-games-part-11/
        - http://software.intel.com/en-us/articles/fluid-simulation-for-video-games-part-12/
        - http://software.intel.com/en-us/articles/fluid-simulation-for-video-games-part-13/
        - http://software.intel.com/en-us/articles/fluid-simulation-for-video-games-part-14/
        - http://software.intel.com/en-us/articles/fluid-simulation-for-video-games-part-15/

        - http://www.gamasutra.com/view/feature/4164/sponsored_feature_fluid_.php
        - http://www.gamasutra.com/view/feature/4176/sponsored_feature_fluid_.php

    \author Copyright 2009-2012 Dr. Michael Jason Gourlay; All rights reserved.  Contact me at mijagourlay.com for licensing.

        - http://www.mijagourlay.com/

*/
#ifndef INTE_SI_VIS_H
#define INTE_SI_VIS_H

#include "Render/qdMaterial.h"
#include "Render/qdCamera.h"
#include "Render/qdLight.h"
#include "Render/qdModel.h"
#include "Render/particleMaterial.h"
#include "Render/particleRenderer.h"

#include "Particles/Operation/pclOpFindBoundingBox.h"
#include "Particles/Operation/pclOpWind.h"
#include "Particles/Operation/pclOpAdvect.h"
#include "Particles/Operation/pclOpEvolve.h"
#include "Particles/Operation/pclOpAssignScalarFromGrid.h"
#include "Particles/Operation/pclOpPopulateVelocityGrid.h"
#include "Particles/Operation/pclOpEmit.h"
#include "Particles/Operation/pclOpKillAge.h"
#include "Particles/particleSystemManager.h"

#include "VortonFluid/pclOpVortonSim.h"

#include "Impulsion/rbSphere.h"
#include "Impulsion/rbBox.h"

#include "FluidBodySim/fluidBodySim.h"
#include "FluidBodySim/pclOpFluidBodyInteraction.h"



/** Simulation entity that binds a physical object to its render model.
*/
class Entity
{
    public:
        Entity( Impulsion::PhysicalObject * physicalObject , QdModel * renderModel )
            : mPhysicalObject( physicalObject )
            , mRenderModel( renderModel )
        {}

        ~Entity() {}

        void Update()
        {
            mRenderModel->SetPositionOrientation( mPhysicalObject->GetBody()->GetPosition() , mPhysicalObject->GetBody()->GetOrientation() ) ;
        }

        Impulsion::PhysicalObject  *    mPhysicalObject ;
        QdModel                    *    mRenderModel    ;
} ;




struct FluidVortonPclGrpInfo
{
    ParticleGroup               *   mParticleGroup                  ;   ///< Particle group for vortons.
    PclOpEmit                   *   mPclOpEmit                      ;   ///< Emit vortons.
    PclOpVortonSim              *   mPclOpVortonSim                 ;   ///< Update velocity grid due to vorton-based fluid simulation.
    PclOpFluidBodyInteraction   *   mPclOpFluidBodInte              ;   ///< Interact vortons with rigid bodies.
    PclOpPopulateVelocityGrid   *   mPclOpPopulateVelocityGrid      ;   ///< Populate velocity grid from particles.
    PclOpAssignVelocityFromField*   mPclOpAssignVelocityFromField   ;   ///< Assign vorton velocity according to field.
    PclOpEvolve                 *   mPclOpEvolve                    ;   ///< Update particle position & orientation from velocities.
    PclOpWind                   *   mPclOpWind                      ;   ///< Apply wind to vortons.
    PclOpKillAge                *   mPclOpKillAge                   ;   ///< Kill old vortons.
} ;

struct FluidTracerPclGrpInfo
{
    ParticleGroup               *   mParticleGroup                  ;   ///< Particle group for tracers.
    PclOpEmit                   *   mPclOpEmit                      ;   ///< Emit passive tracer particles.
    PclOpFindBoundingBox        *   mPclOpFindBoundingBox           ;   ///< Find bounding box containing all tracers.
    PclOpFluidBodyInteraction   *   mPclOpFluidBodInte              ;   ///< Interact passive tracers with rigid bodies.
    PclOpAssignScalarFromGrid   *   mPclOpAssignDensityFromGrid     ;   ///< Assign density to tracers from grid populated from vortons.
    PclOpAssignScalarFromGrid   *   mPclOpAssignFuelFromGrid        ;   ///< Assign fuel to tracers from grid populated from vortons.
    PclOpAssignScalarFromGrid   *   mPclOpAssignFlameFromGrid       ;   ///< Assign flame to tracers from grid populated from vortons.
    PclOpAssignScalarFromGrid   *   mPclOpAssignSmokeFromGrid       ;   ///< Assign smoke to tracers from grid populated from vortons.
    PclOpAssignVelocityFromField*   mPclOpAssignVelocityFromField   ;   ///< Advect tracers according to velocity field.
    PclOpWind                   *   mPclOpWind                      ;   ///< Apply wind to tracers.
    PclOpEvolve                 *   mPclOpEvolve                    ;   ///< Update particle position & orientation.
    PclOpKillAge                *   mPclOpKillAge                   ;   ///< Kill old passive tracer particles.
} ;




/** Application for interactive simulation and visualization.
*/
class InteSiVis
{
    public:
        /** Grid decoration rendering options.
        */
        enum GridDecorationsE
        {
            GRID_DECO_NONE              ,   ///< Do not render grid.
            GRID_DECO_BOUNDING_BOX      ,   ///< Draw bounding box around domain.
            GRID_DECO_CELLS             ,   ///< Draw grid cells.
            GRID_DECO_POINTS            ,   ///< Draw grid points.
            GRID_DECO_CELLS_AND_POINTS  ,   ///< Draw grid cells and points.
        } ;

        InteSiVis() ;
        ~InteSiVis() ;

        void InitDevice( int * pArgc , char ** argv ) ;
        void InitialConditions( unsigned ic ) ;

        float CameraFocusEmphasis( const Vec3 position ) const ;

    private:
        /** Diagnostic text rendering options.
        */
        enum DiagnosticTextE
        {
            DIAG_TEXT_NONE      ,   ///< Do not render diagnostic text.
            DIAG_TEXT_SUMMARY   ,   ///< Render only summary text (not enough to affect frame rate).
            DIAG_TEXT_FULL      ,   ///< Render all diagnostic text.
        } ;

        /** Which value grid field rendering should represent.
        */
        enum GridFieldE
        {
            GRID_FIELD_VELOCITY             ,
            GRID_FIELD_DENSITY_GRADIENT     ,
            GRID_FIELD_PRESSURE_GRADIENT    ,
        } ;

        /** Vortex particle rendering options.
        */
        enum VortonRenderingE
        {
            VORTON_RENDER_NONE                  ,   ///< Do not render vortex particles.
            VORTON_RENDER_DIAGNOSTIC_PARTICLES  ,   ///< Render only the vortons themselves -- no decorations.  Use diagnostic vorton rendering.
            VORTON_RENDER_SIMPLE_PARTICLES      ,   ///< Render only the vortons themselves -- no decorations.
            VORTON_RENDER_VECTORS               ,   ///< Render line decorations (hedgehogs).
            VORTON_RENDER_PARTICLES_AND_VECTORS ,   ///< Render vortons and their line decorations.
            VORTON_RENDER_PATHLINES             ,   ///< Render vortons and their pathlines.
            VORTON_RENDER_ALL                   ,   ///< Render vortons, their decorations and pathlines.
        } ;

        /** Which property to render, both as text next to each vorton, and as a glyph per vorton.
        */
        enum VortonPropertyE
        {
            VORTON_PROPERTY_POSITION            ,
            VORTON_PROPERTY_VELOCITY            ,
            VORTON_PROPERTY_VORTICITY           ,
            VORTON_PROPERTY_DENSITY             ,
            VORTON_PROPERTY_DENSITY_SPH         ,
            VORTON_PROPERTY_DENSITY_GRADIENT    ,
            VORTON_PROPERTY_PROXIMITY           ,
        } ;

        /** Tracer particle rendering options.
        */
        enum TracerRenderingE
        {
            TRACER_RENDER_NONE  ,   ///< Do not render tracer particles.
            TRACER_RENDER_FUEL  ,   ///< Render only the fuel fraction of tracers.
            TRACER_RENDER_FLAME ,   ///< Render only the flame fraction of tracers.
            TRACER_RENDER_SMOKE ,   ///< Render only the smoke fraction of tracers.
            TRACER_RENDER_FIRE  ,   ///< Render fuel, flame and smoke.
            TRACER_RENDER_DYE   ,   ///< Render only density as a two-color dye.
        } ;

        /** Simulation clock options.
        */
        enum TimeSteppingE
        {
            PAUSE       ,   ///< Pause simulation clock.
            SINGLE_STEP ,   ///< Unpause simulation clock for 1 tick, then pause again.
            PLAY        ,   ///< Advance simulation clock normally.
        } ;

        InteSiVis( const InteSiVis & re) ;                /// Disallow copy construction.
        InteSiVis & operator=( const InteSiVis & re ) ;   /// Disallow assignment

        void            CreateParticleMaterials() ;
        void            CreateMaterials() ;
        void            InitializeRendering() ;
        void            CreateEntities() ;
        void            RenderSky() ;
        void            RenderRigidBodies() ;
        void            SetDiagnosticPropertyValueRangeScale( float & rangeScale ) const ;
        void            RenderVortonDiagnosticText() ;
        void            RenderVortonDiagnosticVectors() ;
        void            RenderVortons() ;
        void            RenderTracers() ;
        void            RenderNonFluidParticles() ;
        void            RenderPathlines( VECTOR< Particle > & Particles ) ;
        void            RenderParticles() ;
        void            RenderSummaryDiagnosticText() ;
        void            GatherAndRecordProfileData() ;
        void            UpdateParticleSystems() ;
        void            UpdateRigidBodies() ;
        const char *    FluidSimulationTechniqueString() const ;
        const char *    GridDecorationString() const ;
        const char *    GridFieldString() const ;
        const char *    VortonRenderingString() const ;
        const char *    VortonPropertyString() const ;
        const char *    TracerRenderingString() const ;

        void            SetTallyDiagnosticIntegrals() ;

        void            KeyboardHandler( unsigned char key , int mouseX , int mouseY ) ;
        void            SpecialKeyHandler ( int key , int modifierKeys , int windowRelativeMouseX , int windowRelativeMouseY ) ;
        void            MouseMotionHandler( int x , int y , int modifierKeys) ;
        void            Idle() ;

        static void     GlutDisplayCallback(void) ;
        static void     GlutReshapeGlutCallback(int width, int height) ;
        static void     GlutSpecialKeyHandler( int key , int windowRelativeMouseX , int windowRelativeMouseY ) ;
        static void     GlutKeyboardHandler (unsigned char key, int x, int y) ;
        static void     GlutMouseMotionHandler( int x , int y ) ;
        static void     GlutIdleCallback() ;
        static void     GlutMouseHandler( int button , int state , int x , int y ) ;
        static void     GlutEntryHandler( int state ) ;

        VECTOR< RbSphere >                    &  GetSpheres()           { return mSpheres ; }
        VECTOR< RbBox >                       &  GetBoxes()             { return mBoxes ; }
        VECTOR< Impulsion::PhysicalObject * > &  GetPhysicalObjects()   { return mPhysicalObjects ; }
        Impulsion::PhysicalObject *              GetPhysicalObjectInFocus() ;

        VECTOR< RbSphere >                      mSpheres                    ;   ///< Spherical physical objects.
        VECTOR< RbBox >                         mBoxes                      ;   ///< Box-shaped physical objects.
        VECTOR< Impulsion::PhysicalObject * >   mPhysicalObjects            ;   ///< Dynamic array of physical object addresses.
        size_t                                  mPhysObjFocus               ;   ///< Index info mPhysicalObjects that currently has user focus.

        ParticleSystemManager                   mPclSysMgr                     ;   ///< Addresses of non-fluid particle systems

        FluidVortonPclGrpInfo       mVortonPclGrpInfo           ;
        FluidTracerPclGrpInfo       mTracerPclGrpInfo           ;

        QdCamera                    mCamera                     ;   ///< Camera for rendering
        VECTOR< QdLight >           mLights                     ;   ///< Lights
        VECTOR< QdModel >           mModels                     ;   ///< Models to render.
        GLuint                      mBallDisplayList            ;   ///< Identifier for ball geometry cached in OpenGL server.
        QdMaterial                  mPhysObjMaterial            ;   ///< Material used to render physical objects
        QdMaterial                  mContainerMaterial          ;   ///< Material used to render container physical objects
        QdMaterial                  mBoundingBoxMaterial        ;   ///< Material used to render bounding box
        QdMaterial                  mPathlineMaterial           ;   ///< Material used to render pathlines
        QdMaterial                  mSkyMaterial                ;   ///< Material used to render sky dome
        ParticleMaterial            mDiagnosticVortonMaterial   ;   ///< Material used to render vortons for diagnostics
        ParticleMaterial            mSimpleVortonMaterial       ;   ///< Material used to render vortons
        ParticleMaterial            mDyeMaterial                ;   ///< Material used to render tracers as dye that depends on density
        ParticleMaterial            mSmokeMaterial              ;   ///< Material used to render tracers as smoke
        ParticleMaterial            mFlameMaterial              ;   ///< Material used to render tracers as flame
        ParticleMaterial            mFuelMaterial               ;   ///< Material used to render tracers as fuel
        ParticleRenderer            mDiagnosticVortonRenderer   ;   ///< Renderer for vortons for diagnostics
        ParticleRenderer            mSimpleVortonRenderer       ;   ///< Renderer for vortons
        ParticleRenderer            mDyeRenderer                ;   ///< Renderer for tracers whose texture depends on density.
        ParticleRenderer            mSmokeRenderer              ;   ///< Renderer for smoke tracers
        ParticleRenderer            mFlameRenderer              ;   ///< Renderer for flame tracers
        ParticleRenderer            mFuelRenderer               ;   ///< Renderer for fuel tracers

        int                         mRenderWindow               ;   ///< Identifier for render window
        int                         mStatusWindow               ;   ///< Identifier for status window

        VECTOR< Entity >            mEntities                   ;   ///< Simulation entities.

        unsigned                    mFrame                      ;   ///< Frame counter
        double                      mTimeNow                    ;   ///< Current virtual time
        float                       mTimeStep                   ;   ///< Amount by which to increment virtual time
        TimeSteppingE               mTimeStepping               ;   ///< How to step virtual time

        int                         mMouseButtons[3]            ;   ///< Mouse buttons pressed
        int                         mModifierKeys               ;   ///< Modifier (shift, control, alt) keys pressed during input event.

        bool                        mInitialized                ;   ///< Whether this application has been initialized
        int                         mScenario                   ;   ///< Which scenario is being simulated now

        // Controls for diagnostic information.
        GridDecorationsE                mGridDecorations            ;   ///< Which decorations to draw, if any.
        GridFieldE                      mGridField                  ;   ///< Which value to render as grid.
        VortonRenderingE                mVortonRendering            ;   ///< How to render vortons.
        VortonPropertyE                 mVortonProperty             ;   ///< Which vorton property to render as text next to each vorton.
        TracerRenderingE                mTracerRendering            ;   ///< How to render tracers.
        DiagnosticTextE                 mDiagnosticText             ;   ///< Which diagnostic text to render, if any.
        bool                            mEmphasizeCameraTarget      ;   ///< Whether to emphasize objects near the camera look-at location.

    #if USE_TBB
        tbb::task_scheduler_init tbb_init ;
    #endif
} ;

// Public variables --------------------------------------------------------------
// Public functions --------------------------------------------------------------

#endif
